//===========================================================================
//  @(#) $Name: cflowd-2-0-b8 $
//  @(#) $Id: CflowdCiscoFlowEngine.hh,v 1.3 1998/09/29 18:16:01 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, cflowd++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, cflowd++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for cflowd++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  cflowd++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with cflowd++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

//---------------------------------------------------------------------------
//  class CflowdCiscoFlowEngine
//---------------------------------------------------------------------------
//  This class abstracts a flow-export engine on a Cisco.  We only
//  use it to track sequence numbers in flow-export (which are
//  per-engine; a VIP can send flow-export just as an RSP can, and
//  will use its own sequence numbers).
//---------------------------------------------------------------------------
class CflowdCiscoFlowEngine
{
public:
  //-------------------------------------------------------------------------
  //                      inline CflowdCiscoFlowEngine()
  //.........................................................................
  //  constructor
  //-------------------------------------------------------------------------
  inline CflowdCiscoFlowEngine()
  {
    this->_seqNumber = 0;
    this->_missedFlows = 0;
    this->_flowsReceived = 0;
  }

  //-------------------------------------------------------------------------
  //                         ~CflowdCiscoFlowEngine()
  //.........................................................................
  //  destructor
  //-------------------------------------------------------------------------
  inline ~CflowdCiscoFlowEngine()
  {
  }
  
  //-------------------------------------------------------------------------
  //                    uint32_t SequenceNumber() const
  //.........................................................................
  //  Returns the current flow-export sequence number for the
  //  flow engine.
  //-------------------------------------------------------------------------
  inline uint32_t  SequenceNumber() const
  {
    return(this->_seqNumber);
  }
  
  //-------------------------------------------------------------------------
  //              uint32_t SequenceNumber(uint32_t seqNumber)
  //.........................................................................
  //  Sets and returns the current flow-export sequence number for the
  //  flow engine.
  //-------------------------------------------------------------------------
  inline uint32_t SequenceNumber(uint32_t seqNumber)
  {
    this->_seqNumber = seqNumber;
    return(this->_seqNumber);
  }

  //-------------------------------------------------------------------------
  //     uint32_t SequenceNumber(uint32_t seqNumber, uint16_t flowCount)
  //.........................................................................
  //  Sets and returns the current flow-export sequence number for the
  //  flow engine.  Also updates the flows received for the engine, as
  //  well as handling the missed flows counter.  This can be used as
  //  a single function call from cflowd to keep track of the sequence
  //  number and missed flows.
  //-------------------------------------------------------------------------
  uint32_t SequenceNumber(uint32_t seqNumber, uint16_t flowCount);
  
  //-------------------------------------------------------------------------
  //                       uint32_t MissedFlows() const                   
  //.........................................................................
  //  Returns the number of missed flows for the flow engine.
  //-------------------------------------------------------------------------
  inline uint32_t MissedFlows() const
  {
    return(this->_missedFlows);
  }

  //-------------------------------------------------------------------------
  //                uint32_t MissedFlows(uint32_t missedFlows)            
  //.........................................................................
  //  Sets and returns the number of missed flows for the flow engine.
  //-------------------------------------------------------------------------
  inline uint32_t MissedFlows(uint32_t missedFlows)
  {
    this->_missedFlows = missedFlows;
    return(this->_missedFlows);
  }

  //-------------------------------------------------------------------------
  //                      uint32_t FlowsReceived() const                  
  //.........................................................................
  //  Returns the number of flows received from the flow engine.
  //-------------------------------------------------------------------------
  inline uint32_t FlowsReceived() const
  {
    return(this->_flowsReceived);
  }

  //-------------------------------------------------------------------------
  //              uint32_t FlowsReceived(uint32_t flowsReceived)          
  //.........................................................................
  //  Sets and returns the number of flows received from the flow
  //  engine.
  //-------------------------------------------------------------------------
  inline uint32_t FlowsReceived(uint32_t flowsReceived)
  {
    this->_flowsReceived = flowsReceived;
    return(this->_flowsReceived);
  }

private:
  uint32_t     _seqNumber;
  uint32_t     _missedFlows;
  uint32_t     _flowsReceived;
};

