//===========================================================================
//  $Name: cflowd-2-0-b8 $
//  $Id: CflowdCiscoFlowInterface.hh,v 1.5 1998/09/21 17:11:38 dwm Exp $
//===========================================================================
//  CAIDA Copyright Notice
//
//  By accessing this software, cflowd++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, cflowd++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for cflowd++ software. You can redistribute it
//  and/or modify it under the terms of the GNU General Public License,
//  v.  2 dated June 1991 which is incorporated by reference herein.
//  cflowd++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual property
//  rights.
//
//  You should have received a copy of the GNU GPL along with cflowd++.
//  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/gpl.html
//
//  or by writing to:
//
//    University of California, San Diego
//
//    SDSC/CAIDA
//    9500 Gilman Dr., MS-0505
//    La Jolla, CA 92093 - 0505  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _CFLOWDCISCOFLOWINTERFACE_HH_
#define _CFLOWDCISCOFLOWINTERFACE_HH_

#include "CflowdAsMatrix.hh"
#include "CflowdNetMatrix.hh"
#include "CflowdPortMatrix.hh"
#include "CflowdProtocolTable.hh"
#include "CflowdInterfaceMatrix.hh"
#include "CflowdNextHopTable.hh"

//---------------------------------------------------------------------------
//  class CflowdCiscoFlowInterface
//---------------------------------------------------------------------------
//  This class abstracts an input interface doing flow-switching
//  on a Cisco.  It is used to hold tabular data for an input interface.
//  In cflowd++, we keep one of these objects per input interface so
//  that tables and reports may be input interface specific.
//---------------------------------------------------------------------------
class CflowdCiscoFlowInterface
{
public:
  //-------------------------------------------------------------------------
  //                    inline uint8_t TableIndex() const                    
  //.........................................................................
  //  Returns the table index bitfield for the interface.  Currently
  //  this is unused, since we keep our table index one level up
  //  in the containment heirarchy (in the CflowdCisco class, i.e.
  //  per-Cisco and not per-interface).
  //-------------------------------------------------------------------------
  inline uint8_t TableIndex() const
  {
    return(this->_tableIndex);
  }
  
  //-------------------------------------------------------------------------
  //              inline uint8_t TableIndex(uint8_t tableIndex)              
  //.........................................................................
  //  Sets and returns the table index bitfield for the interface.
  //  Currently this is unused, since we keep our table index one level
  //  up in the containment heirarchy (in the CflowdCisco class, i.e.
  //  per-Cisco and not per-interface).
  //-------------------------------------------------------------------------
  inline uint8_t TableIndex(uint8_t tableIndex)
  {
    this->_tableIndex = tableIndex;
    return(this->_tableIndex);
  }
  
  //-------------------------------------------------------------------------
  //                 inline CflowdAsMatrix & AsMatrix() const                
  //.........................................................................
  //  Returns a reference to the AS matrix for the interface.
  //-------------------------------------------------------------------------
  inline CflowdAsMatrix & AsMatrix() const
  {
    return(this->_asMatrix);
  }
  
  //-------------------------------------------------------------------------
  //                inline CflowdNetMatrix & NetMatrix() const               
  //.........................................................................
  //  Returns a reference to the net matrix for the interface.
  //-------------------------------------------------------------------------
  inline CflowdNetMatrix & NetMatrix() const
  {
    return(this->_netMatrix);
  }
  
  //-------------------------------------------------------------------------
  //               inline CflowdPortMatrix & PortMatrix() const              
  //.........................................................................
  //  Returns a reference to the port matrix for the interface.
  //-------------------------------------------------------------------------
  inline CflowdPortMatrix & PortMatrix() const
  {
    return(this->_portMatrix);
  }
  
  //-------------------------------------------------------------------------
  //            inline CflowdProtocolTable & ProtocolTable() const           
  //.........................................................................
  //  Returns a reference to the protocol table for the interface.
  //-------------------------------------------------------------------------
  inline CflowdProtocolTable & ProtocolTable() const
  {
    return(this->_protocolTable);
  }

  //-------------------------------------------------------------------------
  //          inline CflowdInterfaceMatrix & InterfaceMatrix() const         
  //.........................................................................
  //  Returns a reference to the interface matrix for the interface.
  //-------------------------------------------------------------------------
  inline CflowdInterfaceMatrix & InterfaceMatrix() const
  {
    return(this->_intfMatrix);
  }

  //-------------------------------------------------------------------------
  //             inline CflowdNextHopTable & NextHopTable() const            
  //.........................................................................
  //  Returns a reference to the IP nexthop table for the interface.
  //-------------------------------------------------------------------------
  inline CflowdNextHopTable & NextHopTable() const
  {
    return(this->_nextHopTable);
  }
  
  //-------------------------------------------------------------------------
  //                 int AddFlow(const CflowdRawFlow & flow)
  //.........................................................................
  //  Adds the contents of a raw flow to the interface's tabular
  //  data.  Always returns 0.
  //-------------------------------------------------------------------------
  int AddFlow(const CflowdRawFlow & flow);
  
  
private:
  uint8_t                        _tableIndex;
  mutable CflowdAsMatrix         _asMatrix;
  mutable CflowdNetMatrix        _netMatrix;
  mutable CflowdPortMatrix       _portMatrix;
  mutable CflowdProtocolTable    _protocolTable;
  mutable CflowdInterfaceMatrix  _intfMatrix;
  mutable CflowdNextHopTable     _nextHopTable;
};

#endif  // _CFLOWDCISCOFLOWINTERFACE_HH_
